'use strict';
var path = require('path');
var GoogleAuth = require('google-auth-library');
var logger = require('../../jslib/logger/command.js');
var utils = require('../../jslib/common/index.js');

var oauth2Client;
var SCOPES = ['https://www.googleapis.com/auth/calendar.readonly'];
var listEventsObj;

function authorize(callback) {
	var credentials = utils.readFile(path.join(__dirname, 'client_secret.json')),
		clientSecret = credentials.installed.client_secret,
		clientId = credentials.installed.client_id,
		redirectUrl = credentials.installed.redirect_uris[0],
		auth_info = JSON.parse(this.options.auth_info),
		access_token = auth_info.access_token,
		refresh_token = auth_info.refresh_token,
		auth = new GoogleAuth();

	oauth2Client = new auth.OAuth2(clientId, clientSecret, redirectUrl);
	oauth2Client.setCredentials({
		access_token: access_token,
		refresh_token: refresh_token,
		expiry_date: (new Date()).getTime() + (1000 * 60 * 60 * 24 * 7)
	});

	callback(oauth2Client);
}

function initListEvents (oauth2Client) {
	var ListEvents = require('./list_events.js');
	listEventsObj = new ListEvents(this.options);
	listEventsObj.listEvents(oauth2Client);
}

function destroy() {
	if (listEventsObj) {
		listEventsObj.destroy();
	}
}

exports = module.exports = function (options) {
	var listCalendars = require('./list_calendars.js');

	this.options = options;
	this.listEvents = authorize.bind(this, initListEvents.bind(this));
	this.listCalendars = authorize.bind(this, listCalendars);
	this.destroy = destroy;
}
