'use strict';
var async = require('async');
var fs = require('fs');
var google = require('googleapis');
var childProcessObj = require('child_process');
var childProcess;

var ICSGenerator = require('../../jslib/evt_adapter/index.js');
var gUtils = require('./utils.js');
var utils = require('../../jslib/common/index.js');
var logger = require('../../jslib/logger/command.js');
var listRepeatEvent = require('./list_repeat_event.js');
var now = new Date();
var timeMin = new Date(now.getTime() - 1000*60*60*24*100).toISOString();
var icsPath = `/tmp/synocal_${ICSGenerator.random4()}.ics`;
var isImporting = false;

var calendar = google.calendar('v3'),
	vevents = '',
	ics = '',
	repeatEventList = [],
	vcalendar,
	tzidList = new Set();

var appendVEvents = function (vevent) {
	vevents += vevent;
};

var listEvents = function (oauth2Client, nextPageToken) {
	var params = {
		auth: oauth2Client,
		calendarId: this.calendarId,
		timeMin: timeMin,
		maxResults: 2000,
		singleEvents: false
	};
	if (nextPageToken) {
		params["pageToken"] = nextPageToken;
	}
	this.updateProgress('listCommonEvents');

	calendar.events.list(params, function(err, response) {
		if (err) {
			logger.debug('The API returned an error: ' + err, response);
			utils.onDestroy();
		}
		vevents += gUtils.handleListEventResponse(response, repeatEventList, tzidList);
		if (response.nextSyncToken !== undefined || response.items.length === 0) {
			this.onListAllCommonEvents(oauth2Client);
			return;
		}
		this.listEvents(oauth2Client, response.nextPageToken);
	}.bind(this));
}

var addTimeZone = function (vcalendar) {
	tzidList.forEach(function(tzid) {
		vcalendar.addSubcomponent(ICSGenerator.getZoneByTZId(tzid).component);
	});
}

var importICS = function(vcalendar) {
	this.updateProgress('importICS');
	fs.writeFileSync(icsPath, vcalendar.toString());
//logger.debug(process.memoryUsage());
	isImporting = true;
	childProcess = childProcessObj.exec(`sudo --user=synocalendar /usr/local/bin/php56 -d open_basedir=/ /var/packages/Calendar/target/davical/scripts/load_calendar.php localhost append ${this.calId} ${icsPath}`, function(err, stdout, stderr) {
		if (err) {
			logger.debug(stdout);
			logger.debug(err);
			utils.onDestroy();
		}
		fs.unlinkSync(icsPath);
		logger.debug('Importing calendar is over!');
		process.exit(0);
	});
}

var updateProgress = function(stage) {
	this.progress['progress'] = stage;
	fs.writeFileSync(this.progressPath, JSON.stringify(this.progress));
	try {
		if(this.reminderd_pid != 0) {
			process.kill(this.reminderd_pid, 'SIGUSR2');
		}
	} catch(e) {
		logger.debug(e);
	}
}

var onListAllCommonEvents = function (oauth2Client) {
	this.updateProgress('listRepeatEvents');
	async.each(repeatEventList, function(iCalUID, done) {
		listRepeatEvent(oauth2Client, iCalUID, this.calendarId, done, appendVEvents, tzidList);
	}.bind(this), function(err) {
		if (err) {
			logger.debug('Not all repeat events returned: ' + err);
		}
		ics = ICSGenerator.formIcal(vevents);
		vcalendar = ICAL.Component.fromString(ics);
		this.addTimeZone(vcalendar);
		this.importICS(vcalendar);
	}.bind(this));
}

var destroy = function() {
	if (fs.existsSync(icsPath)) {
		fs.unlinkSync(icsPath);
	}
	if (isImporting) {
		childProcess.kill();
	}
}

function ListEvents (options) {
	this.calendarId = options.remote_cal_id;
	this.calId = options.cal_id_tmp;
	this.task_rec_id = options.task_rec_id;
	this.reminderd_pid= options.reminderd_pid || 0;
	this.progressDir = '/tmp/synocal_foreign_import/';
	if (!fs.existsSync(this.progressDir)) {
		fs.mkdirSync(this.progressDir);
	}
	this.progressPath = `${this.progressDir}${this.task_rec_id}.resp.json`;
	this.progress = {
		method: 'cal_import',
		task_rec_id: this.task_rec_id
	};
	ICSGenerator.registerDefaultTimezone();

	this.listEvents = listEvents;
	this.onListAllCommonEvents = onListAllCommonEvents;
	this.updateProgress = updateProgress;
	this.importICS = importICS;
	this.addTimeZone = addTimeZone;
	this.appendVEvents = appendVEvents;
	this.destroy = destroy;
}
exports = module.exports = ListEvents;
