'use strict';

var net = require('net');
var logger = require('../../jslib/logger');
var adapter = require('../../jslib/evt_adapter');

var kEvtList = [
	"cal_list_reload",
	"cal_reload",
	"cal_create",
	"cal_update",
	"cal_remove",
	"evt_list_reload",
	"evt_reload_by_calid",
	"evt_create",
	"evt_update",
	"quota_update",
	"evt_remove",
	"setting_update",
	"reload_all",
	"alarm"
];

var bindDataFromDomainSocket = function(connection) {
	var buffer = null,
		curLen = -1,
		totalLen = -1;

	connection.on('data', function(data) {
		console.log("======= BEFORE =======");
		console.log(data.toString());
		while (true) {
			if (-1 === totalLen) {
				totalLen = data.readUInt32BE(0);
				curLen = 0;
				data = data.slice(4);
				buffer = new Buffer(totalLen);
			}
			var remainLen = totalLen - curLen;
			curLen += data.copy(buffer, curLen, 0, remainLen);
			data = data.slice(remainLen);
			if (curLen === totalLen) {
				try {
					connection.emit('request', JSON.parse(buffer.toString()));
				} catch (e) {
					logger.error('bad request: ' + buffer.toString());
					logger.error(e.stack);
					connection.destroy();
					return;
				}
				totalLen = -1;
				curLen = -1;
				buffer = null;
			}
			if (0 === data.length) {
				break;
			}
		}
	});
};

var bindEvent = function(connection, relay_send_io) {
	connection.on('request', requestHandler.bind(this, connection));
	connection.on('toUI', toUIRequestHandler.bind(this, connection, relay_send_io));
	connection.on('toWebapi', toWebapiRequestHandler.bind(this, connection));
};

var requestHandler = function(connection, req) {
	var req,
		evt;
	evt = req.evt || "error";

	if (req.evt && 0 <= kEvtList.indexOf(req.evt)) {
		connection.emit('toUI', req);
	} else {
		connection.emit('toWebapi', req);
	}
};

var toUIRequestHandler = function(connection, relay_send_io, req) {
	var username,
		data,
		evt;
	username = req.username;
	evt = req.evt || "error";

	if (username) {
		data = req.data || {};
		if ("evt_list_reload" === evt) {
			// sending to all clients except sender
			relay_send_io.emit('evt_list_reload');
		} else if ("cal_list_reload" === evt) {
			relay_send_io.emit('cal_list_reload');
		} else if ("quota_update" === evt) {
			relay_send_io.emit(evt, data);
		} else {
			if (username instanceof Array) {
				username.forEach(function(name) {
					relay_send_io.to(name).emit(evt, data);
				})
			} else {
				relay_send_io.to(username).emit(evt, data);
			}
		}
	}
};

var toWebapiRequestHandler = function(connection, req) {
	console.log("======= AFTER =======");
	switch (req.method) {
		case 'reply':
			console.log(adapter.updateForReply(req));
			break;
		case 'set_attach':
			console.log(adapter.updateAttachments(req));
			break;
		case 'create':
		case 'set':
			console.log(adapter.update(req, req.method === 'create'));
			break;
		case 'get_cancel_ics':
			console.log(adapter.getCancelIcs(req));
			break;
		default:
			break;
	}
	connectionSend(connection, req);
};

var connectionSend = function(connection, req) {
	var data = JSON.stringify(req),
		length = Buffer.byteLength(data, 'utf8'),
		buffer = new Buffer(length + 4);

	try {
		buffer.writeUInt32BE(length, 0);
		buffer.write(data, 4, length);
		connection.write(buffer);
	} catch (e) {
		logger.error(`write local service socket error ${JSON.stringify(e)}`);
	}
}

module.exports = function(relay_send_io) {
	return net.createServer(function(connection) {
		bindDataFromDomainSocket(connection);
		bindEvent(connection, relay_send_io);
	});
}

