var assert = require('assert');
var utils = require('../import_parser/google/utils.js');
var fs = require('fs');

describe('Google', function() {
	describe('#formDT()', function() {
		var tzidList = new Set();
		it('all day 2017-01-26 ', function() {
			var gTime = {
				"date": "2017-01-26",
				"timeZone": "Asia/Taipei"
			}
			assert.equal('20170126', utils.formDT(gTime, 'Asia/Tokyo', tzidList));
		});
		it('non all day 2017-01-26T11:00:00+08:00', function() {
			var gTime = {
				"dateTime": "2017-01-26T11:00:00+08:00",
				"timeZone": "Asia/Taipei"
			}
			assert.equal('TZID=Asia/Taipei:20170126T110000', utils.formDT(gTime, 'Asia/Tokyo', tzidList));
		});
		it('non all day, no detail tz 2017-01-26T11:00:00+08:00', function() {
			var gTime = {
				"dateTime": "2017-01-26T11:00:00+08:00"
			}
			assert.equal('TZID=Asia/Tokyo:20170126T110000', utils.formDT(gTime, 'Asia/Tokyo', tzidList));
		});
	});
	describe('#formCommonData', function() {
		var tzidList = new Set();
		it('common event', function(done) {
			fs.readFile('simple_event.json', function processClientSecrets(err, content) {
				if (err) {
					console.log('fail to read' + err);
					done(err);
					return;
				}
				gEvent = JSON.parse(content);
				assert.deepEqual({
					summary: 'Calendar Scrum',
					transp: 'TRANSPARENT',
					dtstart: 'TZID=Asia/Taipei:20170117T100000',
					dtend: 'TZID=Asia/Taipei:20170117T110000',
					description: 'Never delay',
					location: '5A',
					sequence: 0,
					uid: 'a8tli2mqojth0jqthtmm6tpio0',
					is_all_day: false
				},utils.formCommonData(gEvent, 'Asia/Taipei', tzidList));
				done();
			});
		});
	});
	describe('#formSimpleRepeatEvent', function() {
		it('simple repeat event', function() {
			content = fs.readFileSync('simple_repeat_event.json');
			response = JSON.parse(content);
			utils.handleRepeatEventResponse(response);
		});
	});
	describe('#formRepeatEvent', function() {
		it('repeat event', function() {
			content = fs.readFileSync('repeat_event.json');
			response = JSON.parse(content);
			utils.handleRepeatEventResponse(response);
		});
	});
});
