DROP TRIGGER IF EXISTS t_cal ON collection;
DROP TRIGGER IF EXISTS t_cal_share ON property;
DROP TRIGGER IF EXISTS t_cal_property ON property;
DROP TRIGGER IF EXISTS t_event ON caldav_data;
DROP TRIGGER IF EXISTS t_invite_event ON syno_invite_mail_user;
DROP TRIGGER IF EXISTS t_alarm ON calendar_alarm;
DROP TRIGGER IF EXISTS t_alarm ON syno_cal_sharing;
DROP TRIGGER IF EXISTS t_cal_share ON syno_cal_sharing;

--- CAL ---

CREATE OR REPLACE FUNCTION MONITOR_CAL()
RETURNS "trigger" AS $$
DECLARE
	row_count bigint;
BEGIN
	execute 'select count(*) from syno_cal_notify;' into row_count;
	IF row_count > 50000 THEN
		RETURN NULL;
	END IF;
	IF TG_OP = 'DELETE' THEN
		IF TG_TABLE_NAME = 'collection' THEN
			INSERT INTO syno_cal_notify(table_name, cal_davname, op, modify_time, create_time ) VALUES (TG_TABLE_NAME, OLD.dav_name, TG_OP, NOW(), NOW());
		ELSIF TG_TABLE_NAME = 'property' THEN
			INSERT INTO syno_cal_notify(table_name, cal_davname, op, modify_time, create_time) VALUES (TG_TABLE_NAME, OLD.dav_name, TG_OP, NOW(), NOW());
		END IF;
	ELSE
		IF TG_TABLE_NAME = 'collection' THEN
			IF TG_OP = 'UPDATE' THEN
				IF NEW.timezone != OLD.timezone OR NEW.dav_displayname != OLD.dav_displayname THEN
					INSERT INTO syno_cal_notify(table_name, cal_davname, op, modify_time, create_time ) VALUES (TG_TABLE_NAME, NEW.dav_name, TG_OP, NEW.modified, NOW());
				END IF;
			ELSE
				INSERT INTO syno_cal_notify(table_name, cal_davname, op, modify_time, create_time ) VALUES (TG_TABLE_NAME, NEW.dav_name, TG_OP, NEW.modified, NOW());
			END IF;
		ELSIF TG_TABLE_NAME = 'property' THEN
			INSERT INTO syno_cal_notify(table_name, cal_davname, op, modify_time, create_time ) VALUES (TG_TABLE_NAME, NEW.dav_name, TG_OP, NEW.changed_on, NOW());
		END IF;
	END IF;
	RETURN NULL;
END;
$$ LANGUAGE plpgsql;

--- EVT ---

CREATE OR REPLACE FUNCTION MONITOR_EVT()
RETURNS "trigger" AS $$
DECLARE
	collection_name text;
	row_count bigint;
BEGIN
	execute 'select count(*) from syno_cal_notify;' into row_count;
	IF row_count > 50000 THEN
		RETURN NULL;
	END IF;
	IF TG_OP = 'DELETE' THEN
			collection_name := substring(OLD.dav_name from '/.*/(.*)/');
			IF collection_name != '.in' THEN
				INSERT INTO syno_cal_notify(table_name, dav_id, collection_id, op, msg, modify_time, create_time, caldav_data, extra_info) VALUES (TG_TABLE_NAME, OLD.dav_id, OLD.collection_id, TG_OP, OLD.caldav_type, NOW(), NOW(), OLD.caldav_data, OLD.dav_name);
			END IF;
	ELSIF TG_OP = 'UPDATE' AND NEW.collection_id != OLD.collection_id THEN
			collection_name := substring(NEW.dav_name from '/.*/(.*)/');
			IF collection_name != '.in' THEN
				INSERT INTO syno_cal_notify(table_name, dav_id, collection_id, op, msg, modify_time, create_time) VALUES (TG_TABLE_NAME, NEW.dav_id, NEW.collection_id, TG_OP, NEW.caldav_type, NEW.modified, NOW());
				INSERT INTO syno_cal_notify(table_name, dav_id, collection_id, op, msg, modify_time, create_time) VALUES (TG_TABLE_NAME, OLD.dav_id, OLD.collection_id, TG_OP, OLD.caldav_type, OLD.modified, NOW());
				INSERT INTO sync_changes(sync_time, collection_id, sync_status, dav_id, dav_name) VALUES (NOW(), OLD.collection_id, 404, OLD.dav_id, OlD.dav_name);
			END IF;
	ELSE
		collection_name := substring(NEW.dav_name from '/.*/(.*)/');
		IF collection_name != '.in' THEN
			INSERT INTO syno_cal_notify(table_name, dav_id, collection_id, op, msg, modify_time, create_time) VALUES (TG_TABLE_NAME, NEW.dav_id, NEW.collection_id, TG_OP, NEW.caldav_type, NEW.modified, NOW());
		END IF;
	END IF;
	RETURN NULL;
END;
$$ LANGUAGE plpgsql;

--- INVITE EVT ---

CREATE OR REPLACE FUNCTION MONITOR_INVITE_EVT()
RETURNS "trigger" AS $$
DECLARE
	dav_name_ics text;
BEGIN
	--- #1516 delete related invited events ---
	dav_name_ics := '%' || OLD.ical_uid || '\.ics';
	DELETE FROM caldav_data WHERE dav_name LIKE dav_name_ics;
	RETURN NULL;
END;
$$ LANGUAGE plpgsql;

--- ALARM ---

CREATE OR REPLACE FUNCTION MONITOR_ALARM_DB()
RETURNS "trigger" AS $$
DECLARE
	row_count bigint;
BEGIN
	execute 'select count(*) from syno_cal_notify;' into row_count;
	IF row_count > 50000 THEN
		RETURN NULL;
	END IF;
	IF TG_OP = 'INSERT' OR TG_OP = 'UPDATE' THEN
		INSERT INTO syno_cal_notify(table_name, dav_id, op, modify_time, create_time) VALUES (TG_TABLE_NAME, NEW.dav_id, TG_OP, NEW.next_trigger, NOW());
		--PERFORM ON_ALARM_DB_CHANGED();
	END IF;
	RETURN NULL;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION MONITOR_CAL_SHARING()
RETURNS "trigger" AS $$
DECLARE
	__my_op			text;
	row_count bigint;
BEGIN
	execute 'select count(*) from syno_cal_notify;' into row_count;
	IF row_count > 50000 THEN
		RETURN NULL;
	END IF;
	IF TG_OP = 'INSERT' OR TG_OP = 'UPDATE' THEN
		IF NEW.is_delete=true THEN
			__my_op := 'DELETE';
		ELSE
			__my_op := 'INSERT';
		END IF;
		INSERT INTO syno_cal_notify(table_name, collection_id, cal_davname, op, modify_time, create_time, user_principal_id) VALUES (TG_TABLE_NAME, NEW.collection_id, NEW.cal_davname, __my_op, now(), now(), NEW.principal_id);
	END IF;
	RETURN NULL;
END;
$$ LANGUAGE plpgsql;

CREATE CONSTRAINT TRIGGER t_event AFTER INSERT OR UPDATE OR DELETE ON caldav_data
DEFERRABLE INITIALLY DEFERRED
FOR EACH ROW EXECUTE PROCEDURE MONITOR_EVT();
CREATE CONSTRAINT TRIGGER t_invite_event AFTER DELETE ON syno_invite_mail_user
DEFERRABLE INITIALLY DEFERRED
FOR EACH ROW EXECUTE PROCEDURE MONITOR_INVITE_EVT();
CREATE CONSTRAINT TRIGGER t_cal AFTER INSERT OR UPDATE OR DELETE ON collection
DEFERRABLE INITIALLY DEFERRED
FOR EACH ROW EXECUTE PROCEDURE MONITOR_CAL();
CREATE CONSTRAINT TRIGGER t_cal_property AFTER INSERT OR UPDATE OR DELETE ON property
DEFERRABLE INITIALLY DEFERRED
FOR EACH ROW EXECUTE PROCEDURE MONITOR_CAL();
CREATE CONSTRAINT TRIGGER t_alarm AFTER INSERT OR UPDATE ON calendar_alarm
DEFERRABLE INITIALLY DEFERRED
FOR EACH ROW EXECUTE PROCEDURE MONITOR_ALARM_DB();
CREATE CONSTRAINT TRIGGER t_cal_share AFTER INSERT OR UPDATE ON syno_cal_sharing
DEFERRABLE INITIALLY DEFERRED
FOR EACH ROW EXECUTE PROCEDURE MONITOR_CAL_SHARING();
